header {
	package edu.usfca.syndiag;
	import java.awt.*;
	import java.awt.event.*;
	import javax.swing.*;
}

/** A tree grammar set locaiton of each node in the Grammar AST
 *
 *  Jia Zheng
 *  University of San Francisco
 *  Oct. 5, 2003
 */
 
class SetLocation extends TreeParser;

options {
	importVocab = ANTLR;
    codeGenBitsetTestThreshold=999;
}

grammar
    :   classDef // ignore 2nd (lexer) grammar for now
    ;

classDef
    :   #( PARSER rules )
    |   #( TREE_PARSER rules )
    |   #( LEXER rules )
    ;

rules
	{int x = 50;
	 int y = 50;
	 
	}
    :   ( r:rule[x, y]
    		{y += ((GrammarAST)r).getHeight() + ((GrammarAST)r).deltaY;
    		}
    	)+
    ;

rule [int x, int y]
	{GrammarAST rl = (GrammarAST)#rule;
	 ((GrammarAST)rl).setLoc(x, y);
	 x += 40;
	 rl.deltaY = 60;
	}
    :   #( RULE r:RULE_REF
    				{((GrammarAST)r).setLoc(x, y);
    				 int w = ((GrammarAST)r).getWidth();
    				 x += w;
    				}
    		#(BLOCK b:block[x, y, BLOCK, RULE])
    	EOR )
    ;

block [int x, int y, int type, int from]
	{GrammarAST b = (GrammarAST)#block;
	 int w = ((GrammarAST)b).getBlockWidth();
	 int connY = y;
	 int d;
	 //d=40;
	 if ((type == BLOCK) && (from == RULE))
		{d = 40;}
		else
		{d = 20;}
	 ((GrammarAST)b).setLoc(x, y);
	 ((GrammarAST)b).setBlockLeftConnPoint(x-d, y+10);
	}
	
    :   ( a:alternative[x, y, w]
    		{y += ((GrammarAST)a).getHeight() + ((GrammarAST)a).deltaY;
    		} 
    	)+
    	{x = x + w + d;
    	 y = connY + 10;
    	 ((GrammarAST)b).setBlockRightConnPoint(x, y);
    	}
    ;


alternative [int x, int y, int w]
	{int i = 0;}
	:   #( a:ALT
    		    {x += w/2 - (((GrammarAST)a).getWidth())/2;
    		     ((GrammarAST)a).setLoc(x, y);
    		    }
    			(e:element[x, y]
    				{i++;
    				 if (i == 1){
    					((GrammarAST)a).setLeftConnPoint(((GrammarAST)e).getLeftConnPoint()[0],
									 ((GrammarAST)e).getLeftConnPoint()[1]);
				 }
    				    x += ((GrammarAST)e).getWidth() + ((GrammarAST)e).deltaX;
    		         }
    			)* 
    			{((GrammarAST)a).setRightConnPoint(((GrammarAST)e).getRightConnPoint()[0],
							   ((GrammarAST)e).getRightConnPoint()[1]);
    	        }
    	)
    ;

element [int x, int y]
	{GrammarAST e = (GrammarAST)#element;
	 e.setLoc(x, y);
	}
    :   a:atom[x, y]
    		{e.setLeftConnPoint(((GrammarAST)a).getLeftConnPoint()[0],
				    ((GrammarAST)a).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)a).getRightConnPoint()[0],
				     ((GrammarAST)a).getRightConnPoint()[1]);
    		}
    |   #(NOT elt:element[x, y])
    		{e.setLeftConnPoint(((GrammarAST)elt).getLeftConnPoint()[0],
				    ((GrammarAST)elt).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)elt).getRightConnPoint()[0],
				     ((GrammarAST)elt).getRightConnPoint()[1]);
    		}
    |   #(TOKEN_RANGE a1:atom[x, y]{x += ((GrammarAST)a1).getWidth();} a2:atom[x, y])
    		{e.setLeftConnPoint(((GrammarAST)a1).getLeftConnPoint()[0],
				    ((GrammarAST)a1).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)a2).getRightConnPoint()[0],
				     ((GrammarAST)a2).getRightConnPoint()[1]);
    		}
    |   #(CHAR_RANGE CHAR_LITERAL CHAR_LITERAL)
    |   eb:ebnf[x, y]
    		{e.setLeftConnPoint(((GrammarAST)eb).getLeftConnPoint()[0],
				    ((GrammarAST)eb).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)eb).getRightConnPoint()[0],
				     ((GrammarAST)eb).getRightConnPoint()[1]);
    		}
    |   t:tree[x, y]
    		{e.setLeftConnPoint(((GrammarAST)t).getLeftConnPoint()[0],
				    ((GrammarAST)t).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)t).getRightConnPoint()[0],
				     ((GrammarAST)t).getRightConnPoint()[1]);
    		}
    |   #( s:SYNPRED b:block[x, y, SYNPRED, SYNPRED] )
    		{e.setLeftConnPoint(((GrammarAST)b).getBlockLeftConnPoint()[0],
				    ((GrammarAST)b).getBlockLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)b).getBlockRightConnPoint()[0],
				     ((GrammarAST)b).getBlockRightConnPoint()[1]);
    		}
	|   ACTION
    |   SEMPRED
    |   EPSILON
			{e.setLeftConnPoint(x, y+10);
			 e.setRightConnPoint(x, y+10);
			}
    ;

ebnf [int x, int y]
	{GrammarAST e = (GrammarAST)#ebnf;
	 e.setLoc(x, y);
	}
	:   #( BLOCK b1:block[x, y, BLOCK, BLOCK] )
    		{e.setLeftConnPoint(((GrammarAST)b1).getLeftConnPoint()[0],
				    ((GrammarAST)b1).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)b1).getRightConnPoint()[0],
				     ((GrammarAST)b1).getRightConnPoint()[1]);
    		}	
    |   #( OPTIONAL b2:block[x, y, OPTIONAL, OPTIONAL] )
    		{e.setLeftConnPoint(((GrammarAST)b2).getLeftConnPoint()[0],
				    ((GrammarAST)b2).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)b2).getRightConnPoint()[0],
				     ((GrammarAST)b2).getRightConnPoint()[1]);
    		}
    |   #( CLOSURE b3:block[x, y, CLOSURE, CLOSURE] )
    		{e.setLeftConnPoint(((GrammarAST)b3).getLeftConnPoint()[0],
				    ((GrammarAST)b3).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)b3).getRightConnPoint()[0],
				     ((GrammarAST)b3).getRightConnPoint()[1]);
    		}
    |   #( POSITIVE_CLOSURE b4:block[x, y, POSITIVE_CLOSURE, POSITIVE_CLOSURE] )
    		{e.setLeftConnPoint(((GrammarAST)b4).getLeftConnPoint()[0],
				    ((GrammarAST)b4).getLeftConnPoint()[1]);
    		 e.setRightConnPoint(((GrammarAST)b4).getRightConnPoint()[0],
				     ((GrammarAST)b4).getRightConnPoint()[1]);
    		}
    ;

tree [int x, int y]
	:   #( t:TREE_BEGIN
			{((GrammarAST)t).setLoc(x, y);} 
			a:atom[x, y]
				{x += ((GrammarAST)a).getWidth() + ((GrammarAST)a).deltaX;
				 ((GrammarAST)t).setLeftConnPoint(((GrammarAST)a).getLeftConnPoint()[0],
								  ((GrammarAST)a).getLeftConnPoint()[1]);
				} 
				(e:element[x, y]
					{x += ((GrammarAST)e).getWidth() + ((GrammarAST)e).deltaX;
					}
				)*
				{((GrammarAST)t).setRightConnPoint(((GrammarAST)e).getRightConnPoint()[0],
								   ((GrammarAST)e).getRightConnPoint()[1]);}
		)
    ;

atom [int x, int y]
	{GrammarAST a = (GrammarAST)#atom;
	 a.setLoc(x, y);
	 int w = a.getWidth();
	 int h = a.getHeight();
	 a.setLeftConnPoint(x, y + h/2);
	 a.setRightConnPoint(x + w, y + h/2);
	}
    :   r:RULE_REF
    |   t:TOKEN_REF
    |   c:CHAR_LITERAL
    |   s:STRING_LITERAL
    |   wi:WILD_CARD
    ;
